import os
import re
import sys
import copy
import shutil
forge = sys.modules["forge"]

# NOTE: The Houdini version of boost may be different.
# The boost-regex appears to be sensitive and they don't provide the dso.
# Moving aside their include/boost seems to fix regex,
# but it leaves their other boost dso's cross-versioned with system includes.

# NOTE: Houdini ode includes also conflict
# Try moving aside their include/ode.

def setup(module):
	module.cppmap = {}
	test_include = {}
	test_cpp = {}
	test_link = {}
	test_variant = forge.houdini_sources[0][0]

	#NOTE separate obj path for each version

	# force all of FE to use Houdini's version of boost
#	forge.includemap['houdini'] = os.path.join(os.getenv('HT'),'include')

	deplibs = forge.corelibs + [
				"fexDrawDLLib",
				"fexMetaDLLib",
				"fexOperateDLLib",
				"fexSurfaceDLLib",
				"fexWindowLib"	]

	fe_houdini_oplibs = ''

	if os.path.exists('ext/grass'):
		fe_houdini_oplibs += ':fexGrassDL'
		sys.stdout.write("grass ")
	if os.path.exists('ext/vegetation'):
		fe_houdini_oplibs += ':fexVegetationDL'
		sys.stdout.write("vegetation ")

	for houdini_source in forge.houdini_sources:
		if not os.path.exists(houdini_source[1]) or not os.path.exists(houdini_source[2]):
			sys.stdout.write("-" + houdini_source[0] + " ")
			continue

		if houdini_source[0] == "":
			sys.stdout.write("HT ")
		else:
			sys.stdout.write(houdini_source[0] + " ")

		srcList = [	"houdini.pmh",
					"HoudiniBrush",
					"HoudiniDraw",
					"HoudiniSOP",
					"PortalOp",
					"SurfaceAccessibleHoudini",
					"SurfaceAccessorHoudini",
					"SurfaceCurvesHoudini",
					"SurfaceTrianglesHoudini",
					"assertHoudini",
					"houdiniDL" ]

		variant = houdini_source[0]
		if variant != test_variant:
			variantPath = module.modPath + '/' + variant
			if os.path.lexists(variantPath) == 0:
				os.symlink('.', variantPath)

			srcListVariant = []
			for src in srcList:
				srcListVariant += [ variant + '/' + src ]
			srcList = srcListVariant

		variantSplit = variant.split('.')
		if len(variantSplit) and variantSplit[0] != "":
			houdini_major = int(variantSplit[0])
		else:
			houdini_major = 12
			houdini_major_string = os.getenv('HOUDINI_MAJOR_RELEASE')
			if houdini_major_string:
				houdini_major = int(houdini_major_string)

		dllname = "fexHoudiniDL" + variant

		dll = module.DLL( dllname, srcList )

		fe_houdini_libs = dllname + fe_houdini_oplibs

		for src in srcList:
			relativePath = forge.RelativePath(forge.rootPath,module.modPath)
			modObjPath = os.path.join(forge.objPath, relativePath)
			objName = modObjPath + '/' + src
			if re.compile(r'(.*)\.pmh$').match(src):
				objName += '.gch'
			else:
				objName += forge.objsuf
			srcTarget = forge.FindTarget(objName)
			srcTarget.includemap = { 'houdini_src' : houdini_source[1] }

			# TEMP until houdini supplies openvdb.h
			if forge.openvdb_install != '':
				srcTarget.includemap[ 'vdb' ] = os.path.join(module.modPath,'..','..', forge.openvdb_install)

			srcTarget.cppmap = { 'houdini_src' :
					' -DFE_HOUDINI_MAJOR=' + str(houdini_major) }
			srcTarget.cppmap['houdini_src'] += ' -DFE_HOUDINI_LIBS=' + fe_houdini_libs
			if variant == test_variant:
				test_include = copy.deepcopy(srcTarget.includemap)
				test_cpp = copy.deepcopy(srcTarget.cppmap)

		# this overrides previous -W flags in codegen
		module.cppmap = {}
		module.cppmap['houdini'] = ' -Wno-deprecated -Wall -W -Wno-parentheses -Wno-sign-compare -Wno-reorder -Wno-uninitialized -Wunused -Wno-unused-parameter'
		module.cppmap['houdini'] += ' -DFE_HOUDINI_DEFAULT_URL_ROOT=\\\"' + forge.houdini_url_root + '\\\"'
		module.cppmap['houdini'] += ' -DFE_HOUDINI_DEFAULT_ICON_ROOT=\\\"' + forge.houdini_icon_root + '\\\"'

		dll.linkmap = {}
		if forge.fe_os == "FE_LINUX":
			# needed for gcc 4.2, not for gcc 4.4, but not sure otherwise
			if forge.compiler_version < 4.3:
				module.cppmap['houdini'] += ' -DNEED_SPECIALIZATION_STORAGE'

			dll.linkmap["houdinilibs"] = " -Xlinker -rpath='$ORIGIN/../../'"
			dll.linkmap["houdinilibs"] += " -shared"
			dll.linkmap["houdinilibs"] += " -L" + houdini_source[2];
#			dll.linkmap["houdinilibs"] += " -lHoudiniUI -lHoudiniOPZ -lHoudiniOP3 -lHoudiniOP2 -lHoudiniOP1 -lHoudiniSIM -lHoudiniGEO -lHoudiniPRM -lHoudiniUT -lHoudiniAPPS3"
		elif forge.fe_os == "FE_WIN32" or forge.fe_os == "FE_WIN64":
			dll.linkmap["houdinilibs"] = "TODO"

		if variant == test_variant:
			test_link = copy.deepcopy(dll.linkmap)

		forge.deps( [ dllname + "Lib" ], deplibs )

		# make sym links for houdini to look at
		# TODO should work on non-linux
		houdiniDso = 'lib' + dllname + '.so'
		houdiniPath = os.path.join(forge.libPath, 'houdini') + variant
		houdiniDsoPath = os.path.join(houdiniPath, 'dso')
		houdiniIconPath = os.path.join(houdiniPath, 'config', 'Icons')
		houdiniToolbarPath = os.path.join(houdiniPath, 'toolbar')
		if os.path.isdir(houdiniDsoPath) == 0:
			os.makedirs(houdiniDsoPath, 0755)
		if os.path.isdir(houdiniIconPath) == 0:
			os.makedirs(houdiniIconPath, 0755)
		if os.path.isdir(houdiniToolbarPath) == 0:
			os.makedirs(houdiniToolbarPath, 0755)

		source = os.path.join('..', '..', houdiniDso)

		houdiniPathLink = os.path.join(forge.libPath, 'houdini') + variant
		houdiniDsoPathLink = os.path.join(houdiniPathLink, 'dso')

		dest = os.path.join(houdiniDsoPathLink, houdiniDso)
		if os.path.lexists(dest) == 0:
			os.symlink(source, dest)

		for icon in ['FE.svg', 'FE_alpha.svg', 'FE_beta.svg']:
			source = os.path.join('..','..','..','..','..','doc','image',icon)
			dest = os.path.join(houdiniIconPath, icon)
			if os.path.lexists(dest) == 0:
				os.symlink(source, dest)

		shelf = os.path.join(houdiniToolbarPath, 'FreeElectron.shelf')
		shutil.copyfile('ext/houdini/FreeElectron.shelf', shelf)

	test = module.Module('test')
	test.includemap = test_include
	test.cppmap = copy.deepcopy(module.cppmap)
	if 'houdini_src' in test_cpp:
		test.cppmap['houdini'] += test_cpp['houdini_src']
	test.linkmap = copy.deepcopy(test_link)
	test_deplibs =	forge.corelibs + [
				"fexHoudiniDL" + test_variant + "Lib" ]

	tests = [ 'xHoudiniContext' ]
	for t in tests:
		test_exe = test.Exe(t)
		forge.deps([t + "Exe"], test_deplibs)

def auto(module):
	test_file = """
#include <SOP/SOP_API.h>

int main(void)
{
	return(0);
}
    \n"""

	houdini_path = os.getenv('HT')
	if houdini_path:
		forge.includemap['houdini'] = os.path.join(houdini_path,'include')

	result = forge.cctest(test_file)

	forge.includemap['houdini'] = ''

	return result
